from pandas import errors as pd_err
from pytz import exceptions as pytz_err


# Base Exceptions ---------------------------------------------------------------------------------
class cyTimesError(Exception):
    """The base error for the cyTimes package."""


class cyTimesTypeError(cyTimesError, TypeError):
    """The base TypeError for the cyTimes package."""


class cyTimesValueError(cyTimesError, ValueError):
    """The base ValueError for the cyTimes package."""


# Parser Exceptions -------------------------------------------------------------------------------
class ParserError(cyTimesError):
    """The base error for the Parser module."""


class ParserFailedError(ParserError, cyTimesValueError):
    """Error for failed parsing"""


class ParserBuildError(ParserFailedError):
    """Error for failed building from result"""


# . configs
class ParserConfigsError(ParserError):
    """Error for the 'parser.Configs' when the settings are invalid."""


class InvalidConfigsValue(ParserConfigsError, cyTimesValueError):
    """Error for the 'parser.Configs' when the value is invalid."""


class InvalidParserInfo(InvalidConfigsValue, cyTimesTypeError):
    """Error for Configs importing invalid 'dateutil.parser.parserinfo'."""


# Pydt/Pddt Exceptions ----------------------------------------------------------------------------
class DatetimeError(cyTimesError):
    """The base error for the datetime module."""


class PydtError(DatetimeError):
    """The base error for the pydt module."""


class PddtError(DatetimeError):
    """The base error for the pddt module."""


class InvalidArgumentError(PydtError, PddtError, cyTimesValueError):
    """Error for invalid arguments."""


class InvalidTypeError(InvalidArgumentError, cyTimesTypeError):
    """Error for invalid type."""


class InvalidTimezoneError(InvalidArgumentError):
    """Error for invalid timezone value."""


class AmbiguousTimeError(InvalidArgumentError, pytz_err.AmbiguousTimeError):
    """Error for ambiguous time."""


class IncomparableError(InvalidTimezoneError, InvalidTypeError):
    """Error for incomparable datetime objects."""


class InvalidTimeUnitError(InvalidArgumentError):
    """Error for invalid time unit value."""


class InvalidFspError(InvalidArgumentError):
    """Error for invalid fractional seconds precision value."""


class InvalidMonthError(InvalidArgumentError):
    """Error for invalid month value."""


class InvalidWeekdayError(InvalidArgumentError):
    """Error for invalid weekday value."""


class OutOfBoundsDatetimeError(InvalidArgumentError, pd_err.OutOfBoundsDatetime):
    """Error for 'dtsobj' that has datetimes out of bounds."""



import datetime
from dateutil.parser import parserinfo

# Configs
class Configs:
    def __init__(self, year1st: bool = False, day1st: bool = False) -> None: ...
    @classmethod
    def from_parserinfo(cls, info: parserinfo) -> Configs: ...
    # Property
    @property
    def year1st(self) -> bool: ...
    @property
    def day1st(self) -> bool: ...
    # Pertain
    @property
    def pertain(self) -> set[str]: ...
    def add_pertain(self, *words: str) -> None: ...
    def rem_pertain(self, *words: str) -> None: ...
    def set_pertain(self, *words: str) -> None: ...
    # Jump
    @property
    def jump(self) -> set[str]: ...
    def add_jump(self, *words: str) -> None: ...
    def rem_jump(self, *words: str) -> None: ...
    def set_jump(self, *words: str) -> None: ...
    # UTC
    @property
    def utc(self) -> set[str]: ...
    def add_utc(self, *words: str) -> None: ...
    def rem_utc(self, *words: str) -> None: ...
    def set_utc(self, *words: str) -> None: ...
    # Timezone
    @property
    def tz(self) -> dict[str, int]: ...
    def add_tz(
        self, word: str, hour: int = 0, minute: int = 0, seconds: int = 0
    ) -> None: ...
    def rem_tz(self, *words: str) -> None: ...
    def set_tz(self, **words_and_offsets: int) -> None: ...
    # Month
    @property
    def month(self) -> dict[str, int]: ...
    def add_month(self, month: int, *words: str) -> None: ...
    def rem_month(self, *words: str) -> None: ...
    def set_month(self, **words_and_months: int) -> None: ...
    # Weekday
    @property
    def weekday(self) -> dict[str, int]: ...
    def add_weekday(self, weekday: int, *words: str) -> None: ...
    def rem_weekday(self, *words: str) -> None: ...
    def set_weekday(self, **words_and_weekdays: int) -> None: ...
    # HMS flag
    @property
    def hms_flag(self) -> dict[str, int]: ...
    def add_hms_flag(self, hms_flag: int, *words: str) -> None: ...
    def rem_hms_flag(self, *words: str) -> None: ...
    def set_hms_flag(self, **words_and_hmsflags: int) -> None: ...
    # AM/PM flag
    @property
    def ampm_flag(self) -> dict[str, int]: ...
    def add_ampm_flag(self, ampm_flag: int, *words: str) -> None: ...
    def rem_ampm_flag(self, *words: str) -> None: ...
    def set_ampm(self, **words_and_ampmflags: int) -> None: ...
    # Special methods
    def __repr__(self) -> str: ...

# Timelex
def timelex(dtstr: str) -> list[str]: ...

# Parser
class Parser:
    def __init__(self, cfg: Configs | None = None) -> None: ...
    # Parse
    def parse(
        self,
        dtstr: str,
        default: datetime.date | datetime.datetime | None = None,
        year1st: bool | None = None,
        day1st: bool | None = None,
        ignoretz: bool = True,
        isoformat: bool = True,
    ) -> datetime.datetime: ...

def parse(
    dtstr: str,
    default: datetime.date | datetime.datetime | None = None,
    year1st: bool | None = None,
    day1st: bool | None = None,
    ignoretz: bool = True,
    isoformat: bool = True,
    cfg: Configs | None = None,
) -> datetime.datetime: ...
def parse_dtobj(
    dtobj: object,
    default: datetime.date | datetime.datetime | None = None,
    year1st: bool | None = None,
    day1st: bool | None = None,
    ignoretz: bool = True,
    isoformat: bool = True,
    cfg: Configs | None = None,
) -> datetime.datetime: ...



# cython: language_level=3

from cpython cimport datetime

# Constants
cdef tuple WEEKDAY_REPRS

# Utils
cdef bint is_delta(object o) except -1

# Delta
cdef class Delta:
    cdef:
        int _years
        int _months
        int _days
        int _hours
        int _minutes
        int _seconds
        int _microseconds
        int _year
        int _month
        int _day
        int _weekday
        int _hour
        int _minute
        int _second
        int _microsecond
        long long _hashcode
    # Arithmetic: addition
    cdef inline object _add_date(self, object o)
    cdef inline object _add_datetime(self, object o)
    cdef inline Delta _add_delta(self, Delta o)
    cdef inline Delta _add_timedelta(self, object o)
    cdef inline Delta _add_relativedelta(self, object o)
    cdef inline Delta _add_int(self, int o)
    cdef inline Delta _add_float(self, double o)
    # Arithmetic: right addition
    cdef inline Delta _radd_relativedelta(self, object o)
    # Arithmetic: subtraction
    cdef inline Delta _sub_delta(self, Delta o)
    cdef inline Delta _sub_timedelta(self, object o)
    cdef inline Delta _sub_relativedelta(self, object o)
    cdef inline Delta _sub_int(self, int o)
    cdef inline Delta _sub_float(self, double o)
    # Arithmetic: right subtraction
    cdef inline object _rsub_date(self, object o)
    cdef inline object _rsub_datetime(self, object o)
    cdef inline Delta _rsub_timedelta(self, object o)
    cdef inline Delta _rsub_relativedelta(self, object o)
    cdef inline Delta _rsub_int(self, int o)
    cdef inline Delta _rsub_float(self, double o)
    # Arithmetic: multiplication
    cdef inline Delta _mul_int(self, int i)
    cdef inline Delta _mul_float(self, double f)
    # Comparison
    cdef inline bint _eq_delta(self, Delta o) except -1
    cdef inline bint _eq_timedelta(self, object o) except -1
    cdef inline bint _eq_relativedelta(self, object o) except -1



from typing import Literal, Hashable
from pandas._libs import lib
import datetime, numpy as np, pandas as pd
from typing_extensions import Self
from cytimes.parser import Configs

# Pddt
class Pddt(pd.DatetimeIndex):
    def __init__(
        self,
        data: object,
        freq: str | datetime.timedelta | pd.offsets.BaseOffset | None = None,
        tz: datetime.tzinfo | str | None = None,
        ambiguous: np.ndarray | Literal["infer", "raise"] = "raise",
        year1st: bool | None = None,
        day1st: bool | None = None,
        cfg: Configs | None = None,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
        copy: bool = False,
    ) -> Self:
        """A drop-in replacement for Pandas `<'DatetimeIndex'>`
        class, providing additional functionalities for more
        convenient datetime operations.

        :param data `<'object'>`: Datetime-like data to construct with.

        :param freq `<'str/timedelta/BaseOffset/None'>`: The frequency of the 'data', defaults to `None`.
            - `<'str'>` A frequency string (e.g. 'D', 'h', 's', 'ms'), or 'infer' for auto-detection.
            - `<'timedelta'>` A datetime.timedelta instance representing the frequency.
            - `<'BaseOffset'>` A pandas Offset instance representing the frequency.
            - `<'None'>` No specified frequency.

        :param tz `<'str/tzinfo/None'>`: Set the timezone of the index, defaults to `None`.
            - `<'datetime.tzinfo'>` A subclass of `datetime.tzinfo`.
            - `<'str'>` Timezone name supported by the 'Zoneinfo' module, or `'local'` for local timezone.
            - `<'None'>` Timezone-naive.

        :param ambiguous `<'str/ndarray'>`: How to handle ambiguous times, defaults to `'raise'`.
            - `<'str'>` Accepts 'infer' or 'raise' for ambiguous times handling.
            - `<'ndarray'>` A boolean array to specify ambiguous times ('True' for DST time).

        :param year1st `<'bool/None'>`: Interpret the first ambiguous Y/M/D value as year, defaults to `None`.
            If 'year1st=None', use `cfg.year1st` if 'cfg' is specified; otherwise, defaults to `False`.

        :param day1st `<'bool/None'>`: Interpret the first ambiguous Y/M/D values as day, defaults to `None`.
            If 'day1st=None', use `cfg.day1st` if 'cfg' is specified; otherwise, defaults to `False`.

        :param cfg `<'Configs/None'>`: Custom parser configurations, defaults to `None`.

        :param unit `<'str/None'>`: Set the datetime unit of the index, defaults to `None`.
            Supported datetime units: 's', 'ms', 'us', 'ns'.

        :param name `<'Hashable/None'>`: The name assigned to the index, defaults to `None`.

        :param copy `<'bool'>`: Whether to make a copy of the 'data', defaults to `False`.
        """

    def __new__(
        cls,
        data: object,
        freq: str | datetime.timedelta | pd.offsets.BaseOffset | None = None,
        tz: datetime.tzinfo | str | None = None,
        ambiguous: np.ndarray | Literal["infer", "raise"] = "raise",
        year1st: bool | None = None,
        day1st: bool | None = None,
        cfg: Configs | None = None,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
        copy: bool = False,
    ) -> Self: ...
    # Constructor ------------------------------------------------------
    @classmethod
    def date_range(
        cls,
        start: str | datetime.datetime | None = None,
        end: str | datetime.datetime | None = None,
        periods: int | None = None,
        freq: str | datetime.timedelta | pd.offsets.BaseOffset | None = "D",
        tz: datetime.tzinfo | str | None = None,
        normalize: bool = False,
        inclusive: Literal["left", "right", "both", "neither"] = "both",
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def parse(
        cls,
        data: object,
        freq: str | datetime.timedelta | pd.offsets.BaseOffset | None = None,
        tz: datetime.tzinfo | str | None = None,
        ambiguous: np.ndarray | Literal["infer", "raise"] = "raise",
        year1st: bool | None = None,
        day1st: bool | None = None,
        cfg: Configs | None = None,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
        copy: bool = False,
    ) -> Self: ...
    @classmethod
    def now(
        cls,
        tz: datetime.tzinfo | str | None = None,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def utcnow(
        cls,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def today(
        cls,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def combine(
        cls,
        date: datetime.date | str | None = None,
        time: datetime.time | str | None = None,
        tz: datetime.tzinfo | str | None = None,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def fromordinal(
        cls,
        ordinal: int | object,
        tz: datetime.tzinfo | str | None = None,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def fromseconds(
        cls,
        seconds: int | float | object,
        tz: datetime.tzinfo | str | None = None,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def fromicroseconds(
        cls,
        us: int | object,
        tz: datetime.tzinfo | str | None = None,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def fromtimestamp(
        cls,
        ts: int | float | object,
        tz: datetime.tzinfo | str | None = None,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def utcfromtimestamp(
        cls,
        ts: int | float | object,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def fromisoformat(
        cls,
        dtstr: str | object,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def fromisocalendar(
        cls,
        iso: dict | list | tuple | pd.DataFrame,
        year: int | None = None,
        week: int | None = None,
        weekday: int | None = None,
        tz: datetime.tzinfo | str | None = None,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def fromdate(
        cls,
        date: datetime.date | object,
        tz: datetime.tzinfo | str | None = None,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def fromdatetime(
        cls,
        dt: datetime.datetime | object,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def fromdatetime64(
        cls,
        dt64: np.datetime64 | object,
        tz: datetime.tzinfo | str | None = None,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    @classmethod
    def strptime(
        cls,
        dtstr: str | object,
        fmt: str,
        size: int | str | bytes | object = 1,
        unit: Literal["s", "ms", "us", "ns"] | None = None,
        name: Hashable | None = None,
    ) -> Self: ...
    # Convertor --------------------------------------------------------
    def ctime(self) -> pd.Index[str]: ...
    def strftime(self, fmt: str) -> pd.Index[str]:
        """Convert to index of strings according to the given format `<'Index[str]'>`.

        :param format `<'str'>`: The format of the datetime strings.
        """

    def isoformat(self, sep: str = "T") -> pd.Index[str]: ...
    def timedf(self) -> pd.DataFrame: ...
    def utctimedf(self) -> pd.DataFrame: ...
    def toordinal(self) -> pd.Index[np.int64]: ...
    def seconds(self, utc: bool = False) -> pd.Index[np.float64]: ...
    def microseconds(self, utc: bool = False) -> pd.Index[np.int64]: ...
    def timestamp(self) -> pd.Index[np.float64]: ...
    def datetime(self) -> np.ndarray[datetime.datetime]: ...
    def date(self) -> np.ndarray[datetime.date]: ...
    def time(self) -> np.ndarray[datetime.time]: ...
    def timetz(self) -> np.ndarray[datetime.time]: ...
    def to_period(self, freq: str | pd.offsets.BaseOffset | None) -> pd.PeriodIndex:
        """Cast to PeriodIndex at a particular frequency `<PeriodIndex>`."""

    def to_series(
        self,
        index: pd.Index | None = None,
        name: Hashable | None = None,
    ) -> pd.Series: ...
    def to_frame(
        self,
        index: bool = True,
        name: Hashable = lib.no_default,
    ) -> pd.DataFrame: ...
    # Manipulator ------------------------------------------------------
    def replace(
        self,
        year: int = -1,
        month: int = -1,
        day: int = -1,
        hour: int = -1,
        minute: int = -1,
        second: int = -1,
        microsecond: int = -1,
        nanosecond: int = -1,
        tzinfo: datetime.tzinfo | str | None = -1,
    ) -> Self: ...
    # . year
    def to_curr_year(self, month: int | str | None = None, day: int = -1) -> Self: ...
    def to_prev_year(self, month: int | str | None = None, day: int = -1) -> Self: ...
    def to_next_year(self, month: int | str | None = None, day: int = -1) -> Self: ...
    def to_year(
        self,
        offset: int,
        month: int | str | None = None,
        day: int = -1,
    ) -> Self: ...
    # . quarter
    def to_curr_quarter(self, month: int = -1, day: int = -1) -> Self: ...
    def to_prev_quarter(self, month: int = -1, day: int = -1) -> Self: ...
    def to_next_quarter(self, month: int = -1, day: int = -1) -> Self: ...
    def to_quarter(self, offset: int, month: int = -1, day: int = -1) -> Self: ...
    # . month
    def to_curr_month(self, day: int = -1) -> Self: ...
    def to_prev_month(self, day: int = -1) -> Self: ...
    def to_next_month(self, day: int = -1) -> Self: ...
    def to_month(self, offset: int, day: int = -1) -> Self: ...
    # . weekday
    def to_monday(self) -> Self: ...
    def to_tuesday(self) -> Self: ...
    def to_wednesday(self) -> Self: ...
    def to_thursday(self) -> Self: ...
    def to_friday(self) -> Self: ...
    def to_saturday(self) -> Self: ...
    def to_sunday(self) -> Self: ...
    def to_curr_weekday(self, weekday: int | str | None = None) -> Self: ...
    def to_prev_weekday(self, weekday: int | str | None = None) -> Self: ...
    def to_next_weekday(self, weekday: int | str | None = None) -> Self: ...
    def to_weekday(self, offset: int, weekday: int | str | None = None) -> Self: ...
    # . day
    def to_yesterday(self) -> Self: ...
    def to_tomorrow(self) -> Self: ...
    def to_day(self, offset: int) -> Self: ...
    # . date&time
    def normalize(self) -> Self:
        """Set the time fields to midnight (i.e. 00:00:00) `<'Pddt'>`.

        This method is useful in cases, when the time does not matter.
        Length is unaltered. The timezones are unaffected.
        """

    def snap(self, freq: str | pd.offsets.BaseOffset) -> Self: ...
    def to_datetime(
        self,
        year: int = -1,
        month: int = -1,
        day: int = -1,
        hour: int = -1,
        minute: int = -1,
        second: int = -1,
        microsecond: int = -1,
        nanosecond: int = -1,
    ) -> Self: ...
    def to_date(self, year: int = -1, month: int = -1, day: int = -1) -> Self: ...
    def to_time(
        self,
        hour: int = -1,
        minute: int = -1,
        second: int = -1,
        microsecond: int = -1,
        nanosecond: int = -1,
    ) -> Self: ...
    def to_first_of(self, unit: str | Literal["Y", "Q", "M", "W"]) -> Self: ...
    def to_last_of(self, unit: str | Literal["Y", "Q", "M", "W"]) -> Self: ...
    def to_start_of(
        self,
        unit: str | Literal["Y", "Q", "M", "W", "D", "h", "m", "s", "ms", "us", "ns"],
    ) -> Self: ...
    def to_end_of(
        self,
        unit: str | Literal["Y", "Q", "M", "W", "D", "h", "m", "s", "ms", "us", "ns"],
    ) -> Self: ...
    # . round / ceil / floor
    def round(self, unit: Literal["D", "h", "m", "s", "ms", "us", "ns"]) -> Self: ...
    def ceil(self, unit: Literal["D", "h", "m", "s", "ms", "us", "ns"]) -> Self: ...
    def floor(self, unit: Literal["D", "h", "m", "s", "ms", "us", "ns"]) -> Self: ...
    # . fsp (fractional seconds precision)
    def fsp(self, precision: int) -> Self: ...
    # Calendar ---------------------------------------------------------
    # . iso
    def isocalendar(self) -> pd.DataFrame: ...
    def isoyear(self) -> pd.Index[np.int64]: ...
    def isoweek(self) -> pd.Index[np.int64]: ...
    def isoweekday(self) -> pd.Index[np.int64]: ...
    # . year
    @property
    def year(self) -> pd.Index[np.int64]: ...
    def is_leap_year(self) -> pd.Index[bool]: ...
    def is_long_year(self) -> pd.Index[bool]: ...
    def leap_bt_year(self, year: int) -> pd.Index[np.int64]: ...
    def days_in_year(self) -> pd.Index[np.int64]: ...
    def days_bf_year(self) -> pd.Index[np.int64]: ...
    def days_of_year(self) -> pd.Index[np.int64]: ...
    def is_year(self, year: int) -> pd.Index[bool]: ...
    # . quarter
    @property
    def quarter(self) -> pd.Index[np.int64]: ...
    def days_in_quarter(self) -> pd.Index[np.int64]: ...
    def days_bf_quarter(self) -> pd.Index[np.int64]: ...
    def days_of_quarter(self) -> pd.Index[np.int64]: ...
    def is_quarter(self, quarter: int) -> pd.Index[bool]: ...
    # . month
    @property
    def month(self) -> pd.Index[np.int64]: ...
    def days_in_month(self) -> pd.Index[np.int64]: ...
    def days_bf_month(self) -> pd.Index[np.int64]: ...
    def days_of_month(self) -> pd.Index[np.int64]: ...
    def is_month(self, month: str | int) -> pd.Index[bool]: ...
    def month_name(self, locale: str | None = None) -> pd.Index[str]:
        """Return the month names with specified locale `<'Index[str]'>`.

        :param locale `<'str/None'>`: The locale to use for month names, defaults to `None`.
            - Locale determining the language in which to return the month
              name. Default (`None`) is English locale ('en_US.utf8').
            - Use the command locale -a on your terminal on Unix systems to
              find your locale language code.
        """
    # . weekday
    @property
    def weekday(self) -> pd.Index[np.int64]: ...
    def is_weekday(self, weekday: int | str) -> pd.Index[bool]: ...
    # . day
    @property
    def day(self) -> pd.Index[np.int64]: ...
    def is_day(self, day: int) -> pd.Index[bool]: ...
    def day_name(self, locale: str | None = None) -> pd.Index[str]:
        """Return the weekday names with specified locale `<'Index[str]'>`.

        :param locale `<'str/None'>`: The locale to use for weekday names, defaults to `None`.
            - Locale determining the language in which to return the weekday
              name. Default (`None`) is English locale ('en_US.utf8').
            - Use the command locale -a on your terminal on Unix systems to
              find your locale language code.
        """
    # . time
    @property
    def hour(self) -> pd.Index[np.int64]: ...
    @property
    def minute(self) -> pd.Index[np.int64]: ...
    @property
    def second(self) -> pd.Index[np.int64]: ...
    @property
    def millisecond(self) -> pd.Index[np.int64]: ...
    @property
    def microsecond(self) -> pd.Index[np.int64]: ...
    @property
    def nanosecond(self) -> pd.Index[np.int64]: ...
    # . date&time
    def is_first_of(
        self,
        unit: str | Literal["Y", "Q", "M", "W"],
    ) -> pd.Index[bool]: ...
    def is_last_of(
        self,
        unit: str | Literal["Y", "Q", "M", "W"],
    ) -> pd.Index[bool]: ...
    def is_start_of(
        self,
        unit: str | Literal["Y", "Q", "M", "W", "D", "h", "m", "s", "ms", "us", "ns"],
    ) -> pd.Index[bool]: ...
    def is_end_of(
        self,
        unit: str | Literal["Y", "Q", "M", "W", "D", "h", "m", "s", "ms", "us", "ns"],
    ) -> pd.Index[bool]: ...
    # Timezone -----------------------------------------------------------------------------
    @property
    def tz_available(self) -> set[str]: ...
    @property
    def tz(self) -> datetime.tzinfo | None:
        """The timezone information `<'tzinfo/None'>`."""

    @property
    def tzinfo(self) -> datetime.tzinfo | None:
        """The timezone information `<'tzinfo/None'>`."""

    def is_local(self) -> bool: ...
    def is_utc(self) -> bool: ...
    def tzname(self) -> str: ...
    def astimezone(
        self,
        tz: datetime.tzinfo | str | None = None,
        ambiguous: np.ndarray | Literal["infer", "raise"] = "raise",
        nonexistent: (
            datetime.timedelta | Literal["shift_forward", "shift_backward", "raise"]
        ) = "raise",
    ) -> Self: ...
    def tz_localize(
        self,
        tz: datetime.tzinfo | str | None,
        ambiguous: np.ndarray | Literal["infer", "raise"] = "raise",
        nonexistent: (
            datetime.timedelta | Literal["shift_forward", "shift_backward", "raise"]
        ) = "raise",
    ) -> Self: ...
    def tz_convert(self, tz: datetime.tzinfo | str | None) -> Self: ...
    def tz_switch(
        self,
        targ_tz: datetime.tzinfo | str | None,
        base_tz: datetime.tzinfo | str | None = None,
        naive: bool = False,
        ambiguous: np.ndarray | Literal["infer", "raise"] = "raise",
        nonexistent: (
            datetime.timedelta | Literal["shift_forward", "shift_backward", "raise"]
        ) = "raise",
    ) -> Self: ...
    # Values -------------------------------------------------------------------------------
    @property
    def freq(self) -> pd.offsets.BaseOffset | None:
        """Return the frequency object if it's set, otherwise None. `<'BaseOffset/None'>`."""

    @property
    def freqstr(self) -> str | None: ...
    @property
    def inferred_freq(self) -> str | None:
        """Tries to return a string representing a frequency.
        Return `None` if it can't autodetect the frequency.
        """

    @property
    def values_naive(self) -> np.ndarray[np.datetime64]: ...
    def as_unit(self, unit: Literal["s", "ms", "us", "ns"]) -> Self: ...
    # Arithmetic ---------------------------------------------------------------------------
    def add(
        self,
        years: int = 0,
        quarters: int = 0,
        months: int = 0,
        weeks: int = 0,
        days: int = 0,
        hours: int = 0,
        minutes: int = 0,
        seconds: int = 0,
        milliseconds: int = 0,
        microseconds: int = 0,
        nanoseconds: int = 0,
    ) -> Self: ...
    def sub(
        self,
        years: int = 0,
        quarters: int = 0,
        months: int = 0,
        weeks: int = 0,
        days: int = 0,
        hours: int = 0,
        minutes: int = 0,
        seconds: int = 0,
        milliseconds: int = 0,
        microseconds: int = 0,
        nanoseconds: int = 0,
    ) -> Self: ...
    def diff(
        self,
        data: object,
        unit: Literal["Y", "Q", "M", "W", "D", "h", "m", "s", "ms", "us", "ns"],
        absolute: bool = False,
        inclusive: Literal["one", "both", "neither"] = "both",
    ) -> np.ndarray[np.int64]: ...
    def mean(*, skipna: bool = True, axis: int = 0) -> pd.Timestamp:
        """Return the mean of the values `<'Timestamp'>`.

        :param skipna `<'bool'>`: Exclude NA/null values, defaults to `True`.
        :param axis `<'int'>`: The axis to use, defaults to `0`.
        """
    # Comparison ---------------------------------------------------------------------------
    def is_past(self) -> pd.Index[bool]: ...
    def is_future(self) -> pd.Index[bool]: ...
    def closest(self, data: object) -> Self: ...
    def farthest(self, dtobjs: object) -> Self: ...



import datetime, numpy as np
from dateutil.relativedelta import relativedelta
from typing_extensions import overload, Self, TypeVar

# Types
_DateT = TypeVar("_DateT", bound=datetime.date)
_DatetimeT = TypeVar("_DatetimeT", bound=datetime.datetime)
_DeltaT = TypeVar("_DeltaT", bound=datetime.timedelta | Delta | relativedelta)
_NumericT = TypeVar("_NumericT", bound=int | float)

# Delta
class Delta:
    def __init__(
        self,
        years: int = 0,
        quarters: int = 0,
        months: int = 0,
        days: int = 0,
        weeks: int = 0,
        hours: int = 0,
        minutes: int = 0,
        seconds: int = 0,
        milliseconds: int = 0,
        microseconds: int = 0,
        year: int = -1,
        month: int = -1,
        day: int = -1,
        weekday: int = -1,
        hour: int = -1,
        minute: int = -1,
        second: int = -1,
        millisecond: int = -1,
        microsecond: int = -1,
    ) -> None: ...
    # Property: relative delta
    @property
    def years(self) -> int: ...
    @property
    def months(self) -> int: ...
    @property
    def days(self) -> int: ...
    @property
    def hours(self) -> int: ...
    @property
    def minutes(self) -> int: ...
    @property
    def seconds(self) -> int: ...
    @property
    def microseconds(self) -> int: ...
    # Properties: absolute delta
    @property
    def year(self) -> int | None: ...
    @property
    def month(self) -> int | None: ...
    @property
    def day(self) -> int | None: ...
    @property
    def weekday(self) -> int | None: ...
    @property
    def hour(self) -> int | None: ...
    @property
    def minute(self) -> int | None: ...
    @property
    def second(self) -> int | None: ...
    @property
    def microsecond(self) -> int | None: ...
    # Arithmetic: addition
    @overload
    def __add__(self, o: _DatetimeT) -> _DatetimeT: ...
    @overload
    def __add__(self, o: _DateT) -> _DateT: ...
    @overload
    def __add__(self, o: _DeltaT) -> Self: ...
    @overload
    def __add__(self, o: np.datetime64) -> datetime.datetime: ...
    @overload
    def __add__(self, o: np.timedelta64) -> Self: ...
    @overload
    def __add__(self, o: _NumericT) -> Self: ...
    # Arithmetic: right addition
    @overload
    def __radd__(self, o: _DatetimeT) -> _DatetimeT: ...
    @overload
    def __radd__(self, o: _DateT) -> _DateT: ...
    @overload
    def __radd__(self, o: _DeltaT) -> Self: ...
    @overload
    def __radd__(self, o: _NumericT) -> Self: ...
    # Arithmetic: subtraction
    @overload
    def __sub__(self, o: _DeltaT) -> Self: ...
    @overload
    def __sub__(self, o: np.timedelta64) -> Self: ...
    @overload
    def __sub__(self, o: _NumericT) -> Self: ...
    # Arithmetic: right subtraction
    @overload
    def __rsub__(self, o: _DatetimeT) -> _DatetimeT: ...
    @overload
    def __rsub__(self, o: _DateT) -> _DateT: ...
    @overload
    def __rsub__(self, o: _DeltaT) -> Self: ...
    @overload
    def __rsub__(self, o: _NumericT) -> Self: ...
    # Arithmetic: multiplication
    @overload
    def __mul__(self, o: _NumericT) -> Self: ...
    @overload
    def __rmul__(self, o: _NumericT) -> Self: ...
    # Arithmetic: division
    @overload
    def __truediv__(self, o: _NumericT) -> Self: ...
    # Arithmetic: negation
    def __neg__(self) -> Self: ...
    # Arithmetic: absolute
    def __abs__(self) -> Self: ...
    # Comparison
    def __eq__(self, o: object) -> bool: ...
    def __bool__(self) -> bool: ...
    # Representation
    def __repr__(self) -> str: ...
    def __hash__(self) -> int: ...



# cython: language_level=3

# Python imports
from datetime import timezone
from zoneinfo import ZoneInfo
from dateutil.parser import parserinfo
from dateutil.relativedelta import relativedelta

# Constants -------------------------------------------------------------------------
# . native types
ZONEINFO: type[ZoneInfo] = ZoneInfo
TIMEZONE: type[timezone] = timezone
# . dateutil types
PARSERINFO: type[parserinfo] = parserinfo
RELATIVEDELTA: type[relativedelta] = relativedelta
